from moodle.base.general import GeneralTypeValue


class CohortType(GeneralTypeValue):
    """Cohort type

    Args:
        type (str): The name of the field: id
                        (numeric value of cohortid) or idnumber (alphanumeric value of idnumber)
        value (str): The value of the cohort
    """
    pass
