from moodle.base.general import GeneralTypeValue


class UserType(GeneralTypeValue):
    """Cohort Usertype

    Args:
        type (str): The name of the field: id
                        (numeric value of id) or username (alphanumeric value of username)
        value (str): The value of the cohort
    """
    pass
